/*
	PluginManager v4
	Copyright (C) 2011, plum

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// header
#include "plugin_mgr.h"

// color
#define RED		0x000000CC
#define GREEN	0x0000CC33
#define WHITE	0xFFFFFFFF
#define GRAY	0x00111111
#define SILVER	0x00606060

// global
int first_clear;
int plugin_count;
PluginList *path_list;

// extern
extern char boot_path[256];
extern BlackList *user_list;
extern SceModuleContainer *mod_list;

// function
void WaitButtons(u32 buttons)
{
	SceCtrlData pad;
	pad.Buttons = buttons;

	while((pad.Buttons & buttons) != 0)
	{
		sceCtrlPeekBufferPositive(&pad, 1);
		sceKernelDelayThread(1000);
	}

	return;
}

int CountModuleList(void)
{
	int count;
	SceModuleContainer *top;

	count = 0;
	top = mod_list;

	while(top != NULL)
	{
		count++;
		top = top->next;
	}

	return count;
}

void FreePluginList(void)
{
	int i;

	for(i = 0; i < plugin_count; i++)
	{
		free_p(path_list[i].path);
	}

	free_p(path_list);
	return;
}

int GetPluginList(void)
{
	SceUID fd;
	char *buf, *path;
	int i, len, match;
	BlackList *user_top;
	SceModuleContainer *mod_top;

	// flag
	switch(boot_path[0])
	{
		case 'm':	{ match = 1; break; }
		case 'e':	{ match = 2; break; }
		default:	return 0;
	}

	// free
	if(plugin_count != 0)
	{
		FreePluginList();
		plugin_count = 0;
	}

	switch(sceKernelInitKeyConfig())
	{
		case PSP_INIT_KEYCONFIG_VSH:	{ path = (match == 1) ? "ms0:/seplugins/vsh.txt" : "ef0:/seplugins/vsh.txt"; break; }
		case PSP_INIT_KEYCONFIG_GAME:	{ path = (match == 1) ? "ms0:/seplugins/game.txt" : "ef0:/seplugins/game.txt"; break; }
		case PSP_INIT_KEYCONFIG_POPS:	{ path = (match == 1) ? "ms0:/seplugins/pops.txt" : "ms0:/seplugins/pops.txt"; break; }
		default:						{ path = NULL; break; }
	}

	fd = sceIoOpen(path, PSP_O_RDONLY, 0);

	if(fd < 0)
		return 0;

	buf = malloc_p(512);

	if(!buf)
	{
		sceIoClose(fd);
		return 0;
	}

	while(CheckEof(fd) == 0)
	{
		ReadLine(fd, buf, 512);

		if(buf[0] != '\0')
		{
			if(strncmp(buf, "ms0", 3) != 0 && strncmp(buf, "ef0", 3) != 0)
				continue;

			len = strlen(buf);

			if(buf[len-1] != '0' && buf[len-1] != '1')
				continue;

			plugin_count++;
		}
	}

	if(plugin_count == 0)
	{
		free_p(buf);
		sceIoClose(fd);
		return 0;
	}

	// reset
	sceIoLseek32(fd, 0, PSP_SEEK_SET);

	// alloc
	path_list = malloc_p(sizeof(PluginList) * plugin_count);

	if(!path_list)
	{
		free_p(buf);
		sceIoClose(fd);
		return 0;
	}

	i = 0;

	while(CheckEof(fd) == 0)
	{
		ReadLine(fd, buf, 512);

		if(buf[0] != '\0')
		{
			if(strncmp(buf, "ms0", 3) != 0 && strncmp(buf, "ef0", 3) != 0)
				continue;

			len = strlen(buf);

			if(buf[len-1] != '0' && buf[len-1] != '1')
				continue;

			path_list[i].stat = buf[len-1] - '0';
			buf[len-1] = '\0';

			// check mod_list
			mod_top = mod_list;

			while(mod_top != NULL)
			{
				if(strstr(buf, mod_top->prxname) != NULL)
				{
					path_list[i].stat = 1;
					break;
				}

				mod_top = mod_top->next;
			}

			// check blacklist
			user_top = user_list;

			while(user_top != NULL)
			{
				if(strstr(buf, user_top->path) != NULL)
				{
					path_list[i].stat = 1;
					break;
				}

				user_top = user_top->next;
			}

			// get path
			path_list[i].path = malloc_p(len);

			if(!path_list[i].path)
				break;

			strcpy(path_list[i].path, buf);
			i++;
		}
	}

	free_p(buf);
	sceIoClose(fd);
	return 1;
}

void StatusDraw(int pos, int change)
{
	int i;
	SceModuleContainer *top;

	i = 0;
	top = mod_list;

	while(top != NULL)
	{
		if(i > 20)
			break;

		// draw
		if(i == pos)
		{
			if(change)
				top->stat ^= 1;

			DrawLine(i+6, GRAY);
			BlitPrintf(3, i+6, GREEN, GRAY, ">%s", top->prxname);
			BlitPrintf(40, i+6, GREEN, GRAY, "[%s]", (top->stat) ? "ON" : "OFF");
		}
		else
		{
			DrawLine(i+6, 0);
			BlitPrint(4, i+6, WHITE, 0, top->prxname);
			BlitPrintf(40, i+6, WHITE, 0, "[%s]", (top->stat) ? "ON" : "OFF");
		}

		i++;
		top = top->next;
	}

	if(i == pos)
	{
		DrawLine(i+6, GRAY);
		BlitPrint(3, i+6, GREEN, GRAY, ">Exit");
	}
	else
	{
		DrawLine(i+6, 0);
		BlitPrint(4, i+6, WHITE, 0, "Exit");
	}

	return;
}

int StatusMenu(void)
{
	int pos, max;
	SceCtrlData pad;
	SceModuleContainer *top;

	// get info
	GetDrawInfo();

	// clear
	ClearDisplay();

	// title
	BlitPrint(1, 0, WHITE, 0, "PluginManager");
	BlitPrint(1, 1, GREEN, 0, "StatusMenu");
	BlitPrintfRight(1, RED, 0, "R->LoaderMenu");

	// under
	PutLine(28, GREEN, 0, '*');

	// adjust
	top = mod_list;

	while(top != NULL)
	{
		DeleteThreadId(top);
		AdjustThreads(top);
		AdjustContainer(top, 1);
		top = top->next;
	}

	// set
	pos = 0;
	max = CountModuleList();

	// check
	if(max > 20)
		max = 20;

	// draw
	StatusDraw(pos, 0);

	// loop
	while(1)
	{
		sceCtrlPeekBufferPositive(&pad, 1);

		if(pad.Buttons & PSP_CTRL_UP)
		{
			pos--;

			if(pos < 0)
				pos = max;

			StatusDraw(pos, 0);
			WaitButtons(PSP_CTRL_UP);
		}
		else if(pad.Buttons & PSP_CTRL_DOWN)
		{
			pos++;

			if(pos > max)
				pos = 0;

			StatusDraw(pos, 0);
			WaitButtons(PSP_CTRL_DOWN);
		}
		else if(pad.Buttons & PSP_CTRL_CROSS)
		{
			StatusDraw(pos, 1);
			WaitButtons(PSP_CTRL_CROSS);

			if(pos == max)
			{
				BlitPrint(1, 29, WHITE, 0, ">Exit...");
				sceKernelDelayThread(700000);
				break;
			}
		}
		else if(pad.Buttons & PSP_CTRL_RTRIGGER)
		{
			WaitButtons(PSP_CTRL_RTRIGGER);
			BlitPrint(1, 29, WHITE, 0, ">LoaderMenu...");
			sceKernelDelayThread(700000);
			return 1;
		}

		sceKernelDelayThread(1000);
	}

	return 0;
}

void LoaderDraw(int pos, int start, int end)
{
	int i, x, count;
	char *ptr;

	x = 0;
	count = start + 15;

	for(i = start; i < count && i < plugin_count; i++)
	{
		ptr = strrchr(path_list[i].path, '/');
		ptr++;

		if(i == pos)
		{
			DrawLine(x+6, GRAY);
			BlitPrintf(3, x+6, RED, GRAY, ">%s", ptr);
			BlitPrint(40, x+6, RED, GRAY, "[Disabled]");
		}
		else
		{
			DrawLine(x+6, 0);

			if(path_list[i].stat == 1)
			{
				BlitPrint(4, x+6, SILVER, 0, ptr);
				BlitPrint(40, x+6, SILVER, 0, "[Enabled]");
			}
			else
			{
				BlitPrint(4, x+6, WHITE, 0, ptr);
				BlitPrint(40, x+6, WHITE, 0, "[Disabled]");
			}
		}

		x++;
	}

	if(plugin_count >= 15)
	{
		if(i == plugin_count && end == plugin_count)
		{
			if(i == pos)
			{
				DrawLine(x+6, GRAY);
				BlitPrint(3, x+6, RED, GRAY, ">Exit");
			}
			else
			{
				DrawLine(x+6, 0);
				BlitPrint(4, x+6, WHITE, 0, "Exit");
			}
		}
	}
	else
	{
		if(i == pos)
		{
			DrawLine(x+6, GRAY);
			BlitPrint(3, x+6, RED, GRAY, ">Exit");
		}
		else
		{
			DrawLine(x+6, 0);
			BlitPrint(4, x+6, WHITE, 0, "Exit");
		}
	}

	return;
}

int LoaderMenu(void)
{
	char *ptr;
	int pos, start, end;
	SceCtrlData pad;

	// get info
	GetDrawInfo();

	// clear
	ClearDisplay();

	// title
	BlitPrint(1, 0, WHITE, 0, "PluginManager");
	BlitPrint(1, 1, RED, 0, "LoaderMenu");
	BlitPrintfRight(1, GREEN, 0, "L->StatusMenu");

	// under
	PutLine(28, RED, 0, '*');

	// set
	pos = 0;
	start = 0;
	end = 14;
	GetPluginList();

	if(plugin_count > 0)
	{
		while(path_list[pos].stat == 1)
		{
			pos++;

			if(pos > end)
			{
				start++;
				end++;
			}

			if(pos >= plugin_count)
				break;
		}
	}

	// draw
	LoaderDraw(pos, start, end);

	// loop
	while(1)
	{
		sceCtrlPeekBufferPositive(&pad, 1);

		if(pad.Buttons & PSP_CTRL_UP)
		{
			if(plugin_count > 0)
			{
				do
				{
					pos--;

					if(pos < start)
					{
						start--;
						end--;
					}

					if(pos < 0)
					{
						pos = plugin_count;

						if(plugin_count >= 15)
						{
							start = plugin_count - 14;
							end = plugin_count;
						}
						else
						{
							start = 0;
							end = 14;
						}

						break;
					}
				}
				while(path_list[pos].stat == 1);
			}

			LoaderDraw(pos, start, end);
			WaitButtons(PSP_CTRL_UP);
		}
		else if(pad.Buttons & PSP_CTRL_DOWN)
		{
			if(plugin_count != 0)
			{
				do
				{
					pos++;

					if(pos > end)
					{
						start++;
						end++;
					}

					if(pos == plugin_count)
						break;

					else if(pos > plugin_count)
					{
						pos = 0;
						start = 0;
						end = 14;
					}
				}
				while(path_list[pos].stat == 1);
			}

			LoaderDraw(pos, start, end);
			WaitButtons(PSP_CTRL_DOWN);
		}
		else if(pad.Buttons & PSP_CTRL_CROSS)
		{
			WaitButtons(PSP_CTRL_CROSS);

			if(pos == plugin_count)
			{
				BlitPrint(1, 29, WHITE, 0, ">Exit...");
				sceKernelDelayThread(700000);
				break;
			}
			else
			{
				// make name
				ptr = strrchr(path_list[pos].path, '/');
				ptr++;

				// get info
				GetDrawInfo();

				// msg
				BlitPrintf(1, 29, WHITE, 0, ">Loading %s...", ptr);

				// load module
				LoadStartModule(path_list[pos].path);
				sceKernelDelayThread(700000);

				// suspend
				SuspendThreads();

				// clear
				ClearDisplay();

				// title
				BlitPrint(1, 0, WHITE, 0, "PluginManager");
				BlitPrint(1, 1, RED, 0, "LoaderMenu");
				BlitPrintfRight(1, GREEN, 0, "L->StatusMenu");

				// under
				PutLine(28, RED, 0, '*');

				// set
				pos = 0;
				start = 0;
				end = 14;
				GetPluginList();

				if(plugin_count > 0)
				{
					while(path_list[pos].stat == 1)
					{
						pos++;

						if(pos > end)
						{
							start++;
							end++;
						}

						if(pos >= plugin_count)
							break;
					}
				}

				// draw
				LoaderDraw(pos, start, end);
			}

			WaitButtons(PSP_CTRL_CROSS);
		}
		else if(pad.Buttons & PSP_CTRL_LTRIGGER)
		{
			WaitButtons(PSP_CTRL_LTRIGGER);
			BlitPrint(1, 29, WHITE, 0, ">StatusMenu...");
			sceKernelDelayThread(700000);
			return 1;
		}

		sceKernelDelayThread(1000);
	}

	// free
	if(plugin_count != 0)
	{
		FreePluginList();
		plugin_count = 0;
	}

	return 0;
}

void MainMenu(void)
{
	int res;

	do
	{
		res = StatusMenu();

		if(res)
		{
			res = LoaderMenu();
		}
	}
	while(res);

	return;
}

int PluginMgrThread(SceSize args, void *argp)
{
	u32 buttons;
	SceCtrlData pad;

	// wait
	while(!sceKernelFindModuleByName("sceKernelLibrary"))
		sceKernelDelayThread(1000);

	buttons = GetConfig(boot_path);

	// main loop
	while(1)
	{
		sceCtrlPeekBufferPositive(&pad, 1);

		if((pad.Buttons & buttons) == buttons)
		{
			// wait
			WaitButtons(buttons);

			// first clear display
			if(first_clear != 1)
			{
				GetDrawInfo();
				ClearDisplay();
				first_clear = 1;
			}

			// suspend
			SuspendThreads();

			// main menu
			MainMenu();

			// resume
			ResumeThreads();
		}

		sceKernelDelayThread(50000);
	}

	return sceKernelExitDeleteThread(0);
}

