/*
	PluginManager v4
	Copyright (C) 2011, plum

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// header
#include <pspkernel.h>
#include <systemctrl.h>
#include <pspsysmem_kernel.h>
#include <pspinit.h>
#include <string.h>

// info
PSP_MODULE_INFO("RebootLoader", 0x1006, 0, 1);

// global
char start_file[256];
char boot_path[256];

// function
int LoadModuleOnReboot(int flag)
{
	SceUID fd, memid;
	int size;
	char *buffer;

	fd = sceIoOpen(boot_path, PSP_O_RDONLY, 0);

	if(fd < 0)
		return -1;

	size = sceIoLseek32(fd, 0, PSP_SEEK_END);
	sceIoLseek32(fd, 0, PSP_SEEK_SET);

	memid = sceKernelAllocPartitionMemory(1, "block", 1, size, NULL);

	if(memid < 0)
	{
		sceIoClose(fd);
		return -1;
	}

	buffer = sceKernelGetBlockHeadAddr(memid);

	sceIoRead(fd, buffer, size);
	sceIoClose(fd);

	sctrlHENLoadModuleOnReboot("/kd/utility.prx", buffer, size, flag);
	return 0;
}

int RebootSystem(char *path)
{
	int api;
	struct SceKernelLoadExecVSHParam param;

	api = sceKernelInitApitype();
	memset(&param, 0, sizeof(param));
	param.size = sizeof(param);
	param.args = strlen(path) + 1;
	param.argp = path;

	switch(api)
	{
		case 0x141:
		case 0x152:
			param.key = "game";
			break;

		case 0x120:
		case 0x123:
		case 0x125:
			param.key = "umdemu";
			break;

		default:
			return -1;
	}

	return sctrlKernelLoadExecVSHWithApitype(api, path, &param);
}

int RebootLoaderThread(SceSize args, void *argp)
{
	int stat = 0;

	switch(sceKernelInitKeyConfig())
	{
		case PSP_INIT_KEYCONFIG_VSH:
		{
			if(LoadModuleOnReboot(BOOTLOAD_VSH) != 1)
			{
				sceKernelExitVSHVSH(NULL);
			}
			break;
		}

		case PSP_INIT_KEYCONFIG_GAME:
		{
			if(LoadModuleOnReboot(BOOTLOAD_GAME | BOOTLOAD_UMDEMU) != -1)
			{
				RebootSystem(start_file);
			}
			break;
		}
	}

	// unload
	sceKernelStopUnloadSelfModule(0, NULL, &stat, NULL);
	return 0;
}

SceUID CreateStartThread(const char *name, SceKernelThreadEntry entry, int pri, int stack)
{
	SceUID thid = sceKernelCreateThread(name, entry, pri, stack, 0, NULL);

	if(thid >= 0)
	{
		sceKernelStartThread(thid, 0, NULL);
	}

	return thid;
}

int module_start(SceSize args, void *argp)
{
	char *ptr;

	if(sceKernelFindModuleByName("PluginManager") != NULL)
		return -1;

	if(sceKernelInitKeyConfig() == PSP_INIT_KEYCONFIG_POPS)
		return -1;

	// make load path
	strcpy(boot_path, (char *)argp);
	ptr = strrchr(boot_path, '/');
	strcpy(ptr + 1, "plugin_mgr.prx");

	// make reboot path
	ptr = sceKernelInitFileName();
	strcpy(start_file, ptr);

	// start thread
	CreateStartThread("RebootLoaderThread", RebootLoaderThread, 16, 0x1000);
	return 0;
}

